==> wrogue-0.7.4.tar.bz2: wrogue-0.7.4/src/game.c <==
/*
 * Copyright (C) 2002-2007 The Warp Rogue Team
 * Part of the Warp Rogue Project
 *
 * This software is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License.
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY.
 *
 * See the license.txt file for more details.
 */

/*
 * Module Name: Game
 * Description: -
 */

#define Uses_Ui
#define Uses_Round
#define Uses_Character
#define Uses_CharacterGeneration
#define Uses_ProgramManager
#define Uses_LoadSave
#define Uses_Area
#define Uses_HallOfFame
#define Uses_Macro
#define Uses_Party
#define Uses_Util
#define Uses_World
#define Uses_WorldGenerator
#define Uses_Options
#define Uses_DataFile
#include "mheader.h"

#include "game.h"


static void		new_game(void);

static void		game_loop(void);

static bool		title_screen(void);

static void		choose_difficulty(void);

static void		intro_screen(void);



static GAME_MODE	GameMode;




/*
 * starts the program
 */
void run_game(void)
{

	game_init();

	while (!title_screen()) {

		/* EMPTY LOOP */;
	}
}



/*
 * returns the current game mode
 */
GAME_MODE game_mode(void)
{

	return GameMode;
}



/*
 * sets the game mode
 */
void game_mode_set(GAME_MODE mode)
{

	GameMode = mode;
}



/*
 * the game controls screen
 */
void game_controls_screen(void)
{
	LIST *items;
	MENU *menu;
	bool terminate_game = false;

	items = list_new();
	list_add(items, "Save and quit");
	list_add(items, "Quit");
	list_add(items, "Macros");
	list_add(items, "Character dump");

	menu = menu_create(items);

	while (true) {

		command_bar_set(4, CM_UP, CM_DOWN, CM_OK, CM_EXIT);

		render_game_controls_screen();

		game_controls_screen_menu(menu);

		if (menu->entered_command == CM_EXIT) break;

		if (menu->highlighted == 0) {

			save_point_create();

			terminate_game = true;

			break;

		} else if (menu->highlighted == 1) {

			terminate_game = true;
			
			break;

		} else if (menu->highlighted == 2) {

			macro_setup_screen();

		} else if (menu->highlighted == 3) {

			character_dump_screen();
		}


	}

	menu_destroy(menu);

	if (terminate_game) {

		game_shutdown();
	}
}



/*
 * the victory sequence
 */
void victory(void)
{

	message(SID_GAME, "You have won! Congratulations! Press Enter.");
	render_game_screen();
	update_screen();

	while (get_key() != KEY_ENTER)
		/* DO NOTHING */;

	if (hall_of_fame_add(player_character(), true)) {

		hall_of_fame_screen();
	}

	character_dump_screen();

	world_erase();

	game_shutdown();
}



/*
 * starts a new game
 */
static void new_game(void)
{

	choose_difficulty();

	create_player_character();

	intro_screen();

	world_generation();

	area_read(pc_spawn_position());

	update_view_character(player_character());

	game_loop();
}



/*
 * the main game loop
 */
static void game_loop(void)
{

	while (true) {

		game_round();
	}
}



/*
 * the title screen
 */
static bool title_screen(void)
{
#define TS_NEW_GAME		0
#define TS_CONTINUE_GAME	1
#define TS_HALL_OF_FAME		2
#define TS_CREDITS		3
#define TS_QUIT			4

	LIST *menu_items;
	MENU *menu;
	bool saved_game;
	int chosen_action;

	saved_game = save_point_exists();

	menu_items = list_new();
	list_add(menu_items, "New game");
	list_add(menu_items, "Continue game");
	list_add(menu_items, "Hall of fame");
	list_add(menu_items, "Credits");
	menu = menu_create(menu_items);

	command_bar_set(4, CM_UP, CM_DOWN, CM_OK, CM_EXIT);

	render_title_screen(false);
	update_screen();

	title_screen_menu(menu);

	if (menu->entered_command == CM_EXIT) {

		chosen_action = TS_QUIT;

	} else {

		chosen_action = menu->highlighted;
	}

	menu_destroy(menu);

	if (chosen_action == TS_NEW_GAME) {

		if (saved_game) {
			CONFIRMATION_DIALOG dialog;

			strcpy(dialog.prompt,
				"You will lose your old game! Sure?"
			);

			dialog.confirmed = false;

			new_game_dialog(&dialog);

			if (!dialog.confirmed) {

				return false;
			}

			world_erase();
		}

		new_game();

	} else if (chosen_action == TS_CONTINUE_GAME) {

		if (!saved_game) {

			render_title_screen("There is no saved game");
			update_screen();

			get_key();

			return false;
		}

		if (!save_point_load()) {

			render_title_screen("Saved game not compatible");
			update_screen();

			get_key();

			return false;
		}

		game_loop();


	} else if (chosen_action == TS_HALL_OF_FAME) {

		hall_of_fame_screen();

		return false;

	} else if (chosen_action == TS_CREDITS) {

		show_text_file("Credits", FILE_CREDITS, false);

		return false;

	} else if (chosen_action == TS_QUIT) {

		game_shutdown();
	}

	return true;

}



/*
 * lets the player choose the difficulty of the game
 */
static void choose_difficulty(void)
{
	LIST *menu_items;
	MENU *menu;

	menu_items = list_new();

	list_add(menu_items, "Normal");
	list_add(menu_items, "Iron man");

	menu = menu_create(menu_items);

	command_bar_set(3, CM_UP, CM_DOWN, CM_OK);
	render_choose_difficulty_screen();
	update_screen();

	choose_difficulty_screen_menu(menu);

	switch (menu->highlighted) {
	case 0:
		game_mode_set(GAME_MODE_NORMAL);
		break;
	case 1:
		game_mode_set(GAME_MODE_IRON_MAN);
		break;
	}

	menu_destroy(menu);
}



/*
 * the intro screen
 */
static void intro_screen(void)
{

	show_text_file("Intro", FILE_INTRO, true);
}

==> wrogue-0.7.5.tar.bz2: wrogue-0.7.5/src/game.c <==
/*
 * Copyright (C) 2002-2007 The Warp Rogue Team
 * Part of the Warp Rogue Project
 *
 * This software is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License.
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY.
 *
 * See the license.txt file for more details.
 */

/*
 * Module Name: Game
 * Description: -
 */

#define Uses_Ui
#define Uses_Round
#define Uses_Character
#define Uses_CharacterGeneration
#define Uses_ProgramManager
#define Uses_LoadSave
#define Uses_Area
#define Uses_Sector
#define Uses_Macro
#define Uses_Party
#define Uses_Util
#define Uses_World
#define Uses_Dungeon
#define Uses_Options
#define Uses_DataFile
#define Uses_Honour
#include "mheader.h"

#include "game.h"


#define FILE_CREDITS			"credits.txt"
#define FILE_INTRO			"intro.txt"


static void		new_game(void);

static void		game_loop(void);

static bool		title_screen(void);

static void		intro_screen(void);




/*
 * starts the program
 */
void run_game(void)
{

	game_init();

	while (!title_screen()) {

		/* EMPTY LOOP */;
	}
}



/*
 * the game controls screen
 */
void game_controls_screen(void)
{
	LIST *items;
	MENU *menu;
	bool terminate_game = false;

	items = list_new();
	list_add(items, "Save and quit");
	list_add(items, "Quit");
	list_add(items, "Macros");
	list_add(items, "Certificate");

	menu = menu_create(items);

	while (true) {

		command_bar_set(4, CM_UP, CM_DOWN, CM_OK, CM_EXIT);

		render_game_controls_screen();

		game_controls_screen_menu(menu);

		if (menu->entered_command == CM_EXIT) break;

		if (menu->highlighted == 0) {

			save_point_create();

			terminate_game = true;

			break;

		} else if (menu->highlighted == 1) {

			world_erase();
			
			terminate_game = true;
			
			break;

		} else if (menu->highlighted == 2) {

			macro_setup_screen();

		} else if (menu->highlighted == 3) {

			certificate_screen(false);
		}


	}

	menu_destroy(menu);

	if (terminate_game) {

		game_shutdown();
	}
}



/*
 * the victory sequence
 */
void victory(void)
{

	message(SID_GAME, "You have won! Congratulations! Press Enter.");
	render_game_screen();
	update_screen();

	while (get_key() != KEY_ENTER)
		/* DO NOTHING */;

	certificate_screen(true);

	world_erase();

	game_shutdown();
}



/*
 * starts a new game
 */
static void new_game(void)
{
	WORLD_POINT starting_position = {12, 7, 34};
	AREA_POINT spawn_point;

	create_player_character();

	intro_screen();

	dungeon_generation();

	area_read(&starting_position);

	sector_elevator(&spawn_point);
	
	if (sector_adjacent(&spawn_point, SC_MOVE_TARGET_SAFE) 
		== NULL) {

		die("*** CORE ERROR *** could not spawn player character"); 
	}

	place_character(player_character(), &spawn_point);

	update_view_character(player_character());

	game_loop();
}



/*
 * the main game loop
 */
static void game_loop(void)
{

	while (true) {

		game_round();
	}
}



/*
 * the title screen
 */
static bool title_screen(void)
{
#define TS_NEW_GAME		0
#define TS_CONTINUE_GAME	1
#define TS_CREDITS		2
#define TS_QUIT			3

	LIST *menu_items;
	MENU *menu;
	bool saved_game;
	int chosen_action;

	saved_game = save_point_exists();

	menu_items = list_new();
	list_add(menu_items, "New game");
	list_add(menu_items, "Continue game");
	list_add(menu_items, "Credits");
	menu = menu_create(menu_items);

	command_bar_set(4, CM_UP, CM_DOWN, CM_OK, CM_EXIT);

	render_title_screen(false);
	update_screen();

	title_screen_menu(menu);

	if (menu->entered_command == CM_EXIT) {

		chosen_action = TS_QUIT;

	} else {

		chosen_action = menu->highlighted;
	}

	menu_destroy(menu);

	if (chosen_action == TS_NEW_GAME) {

		if (saved_game) {
			CONFIRMATION_DIALOG dialog;

			strcpy(dialog.prompt,
				"You will lose your old game! Sure?"
			);

			dialog.confirmed = false;

			new_game_dialog(&dialog);

			if (!dialog.confirmed) {

				return false;
			}

			world_erase();
		}

		new_game();

	} else if (chosen_action == TS_CONTINUE_GAME) {

		if (!saved_game) {

			render_title_screen("There is no saved game");
			update_screen();

			get_key();

			return false;
		}

		if (!save_point_load()) {

			render_title_screen("Saved game not compatible");
			update_screen();

			get_key();

			return false;
		}

		game_loop();

	} else if (chosen_action == TS_CREDITS) {

		show_text_file("Credits", FILE_CREDITS, false);

		return false;

	} else if (chosen_action == TS_QUIT) {

		game_shutdown();
	}

	return true;

}



/*
 * the intro screen
 */
static void intro_screen(void)
{

	show_text_file("Intro", FILE_INTRO, true);
}
